/*
 * Copyright (C) 2000-2001 Peter J Jones (pjones(at)pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

// typesize.cc (a utility to tell you the size of C++ types)

// standard includes
#include <iostream>
#include <iomanip>
#include <string>

// namespace
using namespace std;

// constants
const int k_width = 13;

// function protos
void sizeinfo (size_t bytes);
void endian (void);

// main entry point
int main (int argc, char *argv) {
	cout << "C++ Type Sizes in Bytes (and bits)." << endl;
	
	cout << setw(k_width) << "char: ";	sizeinfo(sizeof(char));
	cout << setw(k_width) << "wchar_t: ";sizeinfo(sizeof(wchar_t));
	cout << setw(k_width) << "bool: ";	sizeinfo(sizeof(bool));
	cout << setw(k_width) << "short: ";	sizeinfo(sizeof(short));
	cout << setw(k_width) << "int: ";	sizeinfo(sizeof(int));
	cout << setw(k_width) << "long: ";	sizeinfo(sizeof(long));
	cout << setw(k_width) << "float: ";	sizeinfo(sizeof(float));
	cout << setw(k_width) << "double: ";sizeinfo(sizeof(double));
	cout << setw(k_width) << "long double: ";sizeinfo(sizeof(long double));
	cout << setw(k_width) << "void*: ";sizeinfo(sizeof(void*));
	cout << setw(k_width) << "size_t: ";sizeinfo(sizeof(size_t));

	endian();

	return 0;
}

// print info about a size
void sizeinfo (size_t bytes) {
	cout << setw(2) << bytes << " (" << bytes * 8 << " bits)" << endl;
}

// print byte order
void endian (void) {
	union {
		short	s;
		char	c[ sizeof(short) ];
	} un;

	un.s = 0x0102;

	cout << "Multibyte Ordering: ";
	if (sizeof(short) == 2) {
		if (un.c[0] == 1 && un.c[1] == 2)
			cout << "Big Endian" << endl;
		else if (un.c[0] == 2 && un.c[1] == 1)
			cout << "Little Endian" << endl;
		else
			cout << "Unknown" << endl;
	} else {
		cout << "Error (this only works on systems with 2 byte shorts)" << endl;
	}
}
